{% extends 'base.html' %}
{% load custom_tags_and_filters %}
{% block title %}{{ event_type }} cancelled{% endblock %}
{% block content %}
	<h3>The {{ event_type|lower }} has been cancelled.</h3>
	<div style="font-size:x-large">
		<p><a href="{% url 'landing' %}"><span class="glyphicon glyphicon-home" style="color:#22b14c"></span> Return to homepage</a></p>
		{% if tool %}<p><a href="{% url 'view_calendar' 'tool' tool.id %}"><span class="glyphicon glyphicon-calendar" style="color:#22b14c"></span> View the {{ tool }} calendar</a></p>{% endif %}
		{% if area %}<p><a href="{% url 'view_calendar' 'area' area.id %}"><span class="glyphicon glyphicon-calendar" style="color:#22b14c"></span> View the {{ area }} calendar</a></p>{% endif %}
	</div>
{% endblock %}