<div class="modal-header">
	<button type="button" class="close" data-dismiss="modal">&times;</button>
	<h4 class="modal-title">{{ title }}</h4>
</div>
<div class="modal-body">
	{{ content }}
</div>
<div class="modal-footer">
	<button class="btn btn-success" data-dismiss="modal">Continue</button>
</div>

<script>
	$('#dialog').on('hidden.bs.modal', redirect);
	function redirect()
	{
		window.location.replace('{{ redirect }}')
	}
</script>