# Generated by Django 2.2.10 on 2020-04-30 17:16

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('NEMO', '0014_version_2_2_0'),
    ]

    operations = [
        migrations.AddField(
            model_name='area',
            name='maximum_capacity',
            field=models.PositiveIntegerField(default=0, help_text='The maximum number of people allowed in this area at any given time. Set to 0 for unlimited.'),
        ),
    ]
