# Generated by Django 2.2.10 on 2020-02-13 14:41

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('NEMO', '0011_version_1_22_0'),
    ]

    operations = [
        migrations.AlterField(
            model_name='tool',
            name='_post_usage_questions',
            field=models.TextField(blank=True, db_column='post_usage_questions', help_text='Upon logging off a tool, questions can be asked such as how much consumables were used by the user. This field will only accept JSON format', null=True),
        ),
    ]
