{% extends 'base.html' %}
{% block title %}No active projects{% endblock %}
{% block content %}
	<h3>Oops</h3>
	<p>You are not a member of any active {{ facility_name }} projects and therefore may not create any reservations. Please contact the {{ facility_name }} staff to enroll in a project.</p>
{% endblock %}