<div>
    <h1 class="text-center">
        <span style="font-size: xx-large;margin-right:10px;" class="glyphicon glyphicon-exclamation-sign danger-highlight"></span>
        Interlock error
    </h1>
    <h3 class="interlock_message text-center"></h3>

    <div class="modal-footer" style="text-align: center">
        <button type="button" class="inter_bypass btn btn-success" style="display: none;">
            <span class="bypass_text"></span>
        </button>
        <button type="button" class="inter_try_again btn btn-primary">Try again</button>
        <button type="button" class="btn btn-warning" onclick="$('#dialog').modal('hide')">Cancel</button>
    </div>
</div>