{% extends 'usage/usage_base.html' %}
{% block usage_content %}
	<div style="margin-top:10px">
		<ul class="nav nav-pills" id="tabs">
			<li class="active"><a>Usage</a></li>
			{% if billing_service %}<li><a href="{{ tab_url }}">Billing Information</a></li>{% endif %}
		</ul>
	</div>

	<div id="content" class="tab-content">
		<div class="tab-pane active" id="usage">
			<br/>
			<p>
				Adjustment activities <b>are not included</b> in the usage information.
			</p>
			{% if no_charges %}
				<h3>There was no usage between {{ start_date|date:"F jS, Y" }} and {{ end_date|date:"F jS, Y" }}.</h3>
			{% endif %}
			{% if missed_reservations %}
				<h3>Missed reservations</h3>
				{% for m in missed_reservations %}
					<div class="alert alert-danger">
						<span style="font-weight:bold">{{ m.reservation_item }}</span><br>
						{% if project_autocomplete or selected_project %}Missed by {{ m.user }}<br>{% endif %}
						{{ m.start }}<br>
						Charged to project {{ m.project }}
					</div>
				{% endfor %}
			{% endif %}
			{% if consumables %}
				<h3>Supplies and consumables</h3>
				{% for c in consumables %}
					<div class="alert alert-info">
						<span style="font-weight:bold">{{ c.consumable }}</span><br>
						{% if project_autocomplete or selected_project %}<span style="font-weight:bold">For {{ c.customer }}</span><br>{% endif %}
						Quantity {{ c.quantity }}<br>
						Purchased from {{ c.merchant }} on {{ c.date }}<br>
						Charged to project {{ c.project }}
					</div>
				{% endfor %}
			{% endif %}
			{% if staff_charges %}
				<h3>Staff charges</h3>
				{% for s in staff_charges %}
					<div class="alert alert-info">
						<span style="font-weight:bold">Work performed by {{ s.staff_member }} {% if project_autocomplete or selected_project %}on behalf of {{ s.customer }}{% else %}on your behalf{% endif %}</span><br>
						{{ s.start }}<br>
						{{ s.end }}<br>
						Charged to project {{ s.project }}
					</div>
				{% endfor %}
			{% endif %}
			{% if training_sessions %}
				<h3>Training sessions</h3>
				{% for t in training_sessions %}
					<div class="alert alert-info">
						{{ t.get_type_display }} training {% if project_autocomplete or selected_project %}for {{ t.trainee }} {% endif %}for {{ t.duration }} minutes taught by <span style="font-weight:bold">{{ t.trainer }}</span>
						for the <span style="font-weight:bold">{{ t.tool }}</span>. Charged to project {{ t.project }} on {{ t.date }}.
					</div>
				{% endfor %}
			{% endif %}
			{% if area_access %}
				<h3>Area access</h3>
				{% for a in area_access %}
					<div class="alert {% if not a.staff_charge %}alert-info{% else %}alert-warning{% endif %}">
						<span style="font-weight:bold">{{ a.area }}</span><br>
						{% if a.staff_charge %}<span style="font-weight:bold">Area accessed by {{ a.staff_charge.staff_member }} {% if project_autocomplete or selected_project %}on behalf of {{ a.customer }}{% else %}on your behalf{% endif %}</span><br>
						{% else %}{% if project_autocomplete or selected_project %}<span style="font-weight:bold">Area accessed by {{ a.customer }}</span><br>{% endif %}
						{% endif %}
						{{ a.start }}<br>
						{{ a.end|default_if_none:"In progress" }}<br>
						Charged to project {{ a.project }}
					</div>
				{% endfor %}
			{% endif %}
			{% if usage_events %}
				<h3>Tool usage</h3>
				{% for u in usage_events %}
					<div class="alert {% if u.user == u.operator %}alert-info{% else %}alert-warning{% endif %}">
						<span style="font-weight:bold">{{ u.tool }}</span><br>
						{% if u.user != u.operator %}<span style="font-weight:bold">Operated by {{ u.operator }} {% if project_autocomplete or selected_project %}on behalf of {{ u.user }}{% else %}on your behalf{% endif %}</span><br>
						{% else %}{% if project_autocomplete or selected_project %}<span style="font-weight:bold">Operated by {{ u.user }}</span><br>{% endif %}
						{% endif %}
						{{ u.start }}<br>
						{{ u.end }}<br>
						Charged to project {{ u.project }}
					</div>
				{% endfor %}
			{% endif %}
		</div>
	</div>
{% endblock %}